## $Id: raw_score_report.py 16693 2021-11-02 17:21:20Z henrik $
##
## Copyright (C) 2013 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import grok
from zope.interface import implementer, Interface, Attribute
from waeup.kofa.students.reports.raw_score_report import (
    RawScoreReport, RawScoreReportGenerator,
    IRawScoreReport, RawScoreReportGeneratorPage,
    RawScoreReportPDFView)


class CustomRawScoreReport(RawScoreReport):

    @property
    def signatures(self):
        site = grok.getSite()
        dean = site['faculties'][self.faccode].officer_1
        hod = site['faculties'][self.faccode][self.depcode].officer_1
        if not dean:
            dean = 'Dean of Faculty'
        if not hod:
            hod = 'Head of Department'
        dean = dean.replace(' - ', '<br />')
        hod = hod.replace(' - ', '<br />')
        return [hod, dean]


class CustomRawScoreReportGenerator(RawScoreReportGenerator):

    def generate(self, site, faccode=None, depcode=None, certcode=None,
                 session=None, level=None, author=None):
        result = CustomRawScoreReport(faccode=faccode, depcode=depcode,
            certcode=certcode, session=session, level=level, author=author)
        return result

# Raw Score Report for graduating students only

class IGradRawScoreReport(IRawScoreReport):
    """Marker interface to distuingish graduating students raw score reports
    from ordinary raw score reports.
    """

@implementer(IGradRawScoreReport)
class GradRawScoreReport(CustomRawScoreReport):
    """
    """

    title = 'RAW Score Report'

    @property
    def right_footer(self):
        return self.title + ' (Graduating Students) - %s -' % self.session

    def _excluded(self, level_obj):
        # Do not list student if selecetd level is not final or an extension
        if level_obj.level < level_obj.student['studycourse'].certificate.end_level:
            return True
        # Filter out students with extension levels higher than selected level
        final_level = int(max(level_obj.student['studycourse'].keys()))
        if final_level > level_obj.level:
            return True
        if level_obj.remark not in (
            'Pass', '3s_rd_s', '2s_2_s', '2s_1_s', '1s_st_s'):
            return True
        return False

class GradRawScoreReportGenerator(RawScoreReportGenerator):

    title = 'RAW Score Report (Graduating Students)'
    grok.name('grad_raw_score_report')

    def generate(self, site, faccode=None, depcode=None, certcode=None,
                 session=None, level=None, author=None):
        result = GradRawScoreReport(faccode=faccode, depcode=depcode,
            certcode=certcode, session=session, level=level, author=author)
        return result

class GradRawScoreReportGeneratorPage(RawScoreReportGeneratorPage):

    grok.context(GradRawScoreReportGenerator)

class GradRawScoreReportPDFView(RawScoreReportPDFView):

    grok.context(IGradRawScoreReport)
    grok.name('pdf')
    grok.require('waeup.handleReports')

    def _filename(self):
        return 'GradRawScoreReport_rno%s_%s.pdf' % (
            self.context.__name__,
            self.context.creation_dt_string)

